unit JoinGame;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls,
  Constants, ConnectToIP, ServerInfo, ServerInfoList, WindowsUtils, LanScanner,
  ComCtrls;

type
  TFrmJoinGame = class(TForm)
    GrpSelectServer: TGroupBox;
    lstServers: TListBox;
    BtnConnectAtIP: TButton;
    GrpServerInfo: TGroupBox;
    lblServerInfoName: TLabel;
    lblShowServerInfoName: TLabel;
    lblServerInfoPasswordRequired: TLabel;
    lblShowServerInfoPasswordRequired: TLabel;
    lblServerInfoSlots: TLabel;
    lblShowServerInfoSlots: TLabel;
    lblServerInfoVersion: TLabel;
    lblShowServerInfoVersion: TLabel;
    lblServerInfoPort: TLabel;
    lblShowServerInfoPort: TLabel;
    lblServerInfoIP: TLabel;
    lblShowServerInfoIP: TLabel;
    lblServerInfoCip: TLabel;
    lblShowServerInfoCip: TLabel;
    BtnOK: TButton;
    txtNickname: TEdit;
    lblNickname: TLabel;
    procedure lstServersClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BtnConnectAtIPClick(Sender: TObject);
    procedure BtnOKClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    ServerList: TServerInfoList;
    procedure ShowServerInfo(ServerInfo: TServerInfo);
  public
    { Public declarations }
  end;

var
  FrmJoinGame: TFrmJoinGame;

implementation

{$R *.dfm}

procedure TFrmJoinGame.lstServersClick(Sender: TObject);
var
  C: integer;
begin
  { Itera per trovare l'elemento della lista selezionato... }
  for C := 0 to lstServers.Items.Count-1 do
  begin
    if lstServers.Selected[C] then begin

      { Una volta trovato lo visualizza }
      ShowServerInfo(TServerInfo(lstServers.Items.Objects[C]));
      BtnOK.Enabled := true;
      exit;
    end;
  end;
end;

{ Procedura per mostrare il contenuto di un oggetto ServerInfo sul riepilogo }
procedure TFrmJoinGame.ShowServerInfo(ServerInfo: TServerInfo);
begin
  lblShowServerInfoName.Caption := ServerInfo.GetServerName;

  if ServerInfo.IsPasswordRequired then lblShowServerInfoPasswordRequired.Caption := 'si'
  else lblShowServerInfoPasswordRequired.Caption := 'no';

  lblShowServerInfoSlots.Caption := IntToStr(ServerInfo.GetPlayersConnected) + '/' + IntToStr(ServerInfo.GetSlots);
  lblShowServerInfoCip.Caption := IntToStr(ServerInfo.GetCip);
  lblShowServerInfoIP.Caption := ServerInfo.Host;
  lblShowServerInfoPort.Caption := IntToStr(ServerInfo.Port);
  lblShowServerInfoVersion.Caption := ServerInfo.GetServerVersion;
end;

procedure TFrmJoinGame.FormCreate(Sender: TObject);
begin
  { Crea la lista di server }
  ServerList := TServerInfoList.Create;

  { Disabilita il pulsante di OK visto che all'inizio non ci sono partite }
  BtnOK.Enabled := false;

  { Setta il nome utente... }
  txtNickname.Text := GetLoginName;
end;

procedure TFrmJoinGame.BtnConnectAtIPClick(Sender: TObject);
var
  ConnectToIP: TFrmConnectToIP;
  ServerInfo: TServerInfo;
begin
  { Richiama la finestra di dialogo  }
  ConnectToIP := TFrmConnectToIP.Create(self);

  if ConnectToIP.ShowModal = mrOK then begin
    { OK, connettiamoci all'Host/IP selezionato }
    ServerInfo := TServerInfo.Create(self,ConnectToIP.txtIP.Text,StrToInt(ConnectToIP.txtPort.Text));

    { E' disponibile il server? }
    if ServerInfo.IsAvaible then begin

      { Visualizza le informazioni nel riepilogo }
      ShowServerInfo(ServerInfo);

      { Aggiunge l'oggetto ServerInfo nella lista }
      ServerList.AddServerInfo(ServerInfo);

      { Riempie la listbox }
      ServerList.FillListBox(lstServers);

      { E abilita il pulsante per entrare... }
      BtnOK.Enabled := true;
    end
    else
      MessageBox(0,pchar('Impossibile connettersi a ' + ConnectToIP.txtIP.Text + ':' + ConnectToIP.txtPort.Text + '. Nessuna partita risulta attiva a quest''indirizzo.'),'Nessuna partita trovata.',MB_OK or MB_ICONINFORMATION);
    end;
end;

procedure TFrmJoinGame.BtnOKClick(Sender: TObject);
begin
  { Continua solamente se  selezionata una partita }
  if lblShowServerInfoName.Caption <> '' then begin
    self.ModalResult := mrOK;
  end;
end;

procedure TFrmJoinGame.FormShow(Sender: TObject);
var
  LanScanner: TLanScanner;
begin
  { Crea un oggetto LanScanner che
  setaccera' la rete locale in cerca di un server attivo }
  LanScanner := TLanScanner.Create(self,lstServers, ServerList);
end;

end.
