unit LobbyClient;

interface

uses Windows, SysUtils, IdTcpClient, IdHTTP, LobbyClientForm, Graphics, Classes, StrUtils,
  Constants, ChatClient, LobbyListChecker, ServerInfoList, ServerInfo, HostGame, Controls,
  ConnectionTest, TestServer;

{ Classe TLobbyGateway }
type
  TLobbyGateway = class
  public
    Host: string;
    Port: integer;

    constructor Create(Host: string; Port: integer);
  end;

{ Classe TLobbyClient }
type
  TLobbyClient = class(TThread)
  private
    GatewayList: TList;
    AOwner: TComponent; //Riferimento ad aowner
    FrmLobbyClient: TFrmLobbyClient;
    ClientSocket: TIdTcpClient;
    Nickname: string;
    Disconnecting: boolean;

    { Variabili per il test di connessione }
    FrmConnectionTest: TFrmConnectionTest;
    IAmAbleToHost: boolean;
    ConnectionTestDone: boolean;
    TestServer: TTestServer;

    procedure ReceiveGatewayList;
    procedure AddGateway(Host: string; Port: string);
    function PopGateway: TLobbyGateway;
    procedure Disconnect;

    { Gestore d'evento per i pulsanti }
    procedure BtnSend_Click(Sender: TObject);
    procedure BtnHost_Click(Sender: TObject);
    procedure BtnJoin_Click(Sender: TObject);

    { Gestore d'evento per la fine del test di connessione }
    procedure ConnectionTest_Finished(Sender: TObject);

    { Gestore d'evento per la connessione }
    procedure ClientSocket_Connected(Sender: TObject);
  public
    constructor Create(AOwner: TComponent; FrmLobbyClient: TFrmLobbyClient; Nickname: string);
    procedure Execute; override;
    destructor Destroy; override;
  end;

implementation

{ Implementazione classe TLobbyGateway }
constructor TLobbyGateway.Create(Host: string; Port: integer);
begin
  self.Host := Host;
  self.Port := Port;
end;

{ Implementazione classe TLobbyClient }
constructor TLobbyClient.Create(AOwner: TComponent; FrmLobbyClient: TFrmLobbyClient; Nickname: string);
begin
  self.AOwner := AOwner;
  self.FrmLobbyClient := FrmLobbyClient;
  self.Nickname := Nickname;
  FrmLobbyClient.Nickname := Nickname;

  { Imposta il gestore d'evento per il pulsante send della chat }
  FrmLobbyClient.BtnSend.OnClick := BtnSend_Click;
  FrmLobbyClient.BtnSend.Enabled := false;

  { E quello per hostare e joinare }
  FrmLobbyClient.BtnHost.OnClick := BtnHost_Click;
  FrmLobbyClient.BtnHost.Enabled := false;
  FrmLobbyClient.BtnJoin.OnClick := BtnJoin_Click;
                                                                                                    
  { Crea la socket }                                                                              
  ClientSocket := TIdTcpClient.Create(AOwner);

  { Imposta i gestori d'evento per la socket }
  ClientSocket.OnConnected := ClientSocket_Connected;

  { Imposta una variabile che serve per vedere se ci stiamo disconnettendo }
  Disconnecting := false;

  { Richiama il costruttore base }
  inherited Create(false);
end;

{ Distruttore }
destructor TLobbyClient.Destroy;
begin
  { Se siamo connessi avvisa che ce ne stiamo andando... }
  Disconnect;

  inherited Destroy;
end;

{ Procedura per prendere un Gateway dalla lista }
function TLobbyClient.PopGateway: TLobbyGateway;
begin
  if GatewayList.Count = 0 then raise Exception.Create('Si  cercato di prendere un gateway dalla lista quando essa era vuota. Contattare il produttore.');
  Result := TLobbyGateway(GatewayList.Items[0]);
  GatewayList.Delete(0);
end;

{ Procedura per disconnettersi }
procedure TLobbyClient.Disconnect;
begin
  if ClientSocket.Connected then begin
    Disconnecting := true;
    ClientSocket.WriteLn('/IDISCONNECT');
    ClientSocket.Disconnect;
  end;
end;

{ Entry Point del thread }
procedure TLobbyClient.Execute;
var
  LobbyGateway: TLobbyGateway;
begin
  { Prima di tutto, riceviamo il file contentente la lista dei gateway disponibili... }
  FrmLobbyClient.ShowStatus('Connessione al server web contenente la lista dei gateway disponibili in corso...',clNavy);
  ReceiveGatewayList;
  //GatewayList := TList.Create;
  //AddGateway('localhost','1501');


  { Proviamo a connetterci ai gateway, uno alla volta con un timeout minimo }
  while (GatewayList.Count > 0) and (not ClientSocket.Connected) and (not Disconnecting) do
  begin
    LobbyGateway := PopGateway;
    ClientSocket.Host := LobbyGateway.Host;
    ClientSocket.Port := LobbyGateway.Port;

    FrmLobbyClient.ShowStatus('Tentativo di connessione a ' + ClientSocket.Host + ':' + IntToStr(ClientSocket.Port));

    try
      { Prova a coonnettersi al gateway selezionato }
      ClientSocket.Connect(GATEWAYTIMEOUT);
    except
      on E: exception do
      begin
        { Gestione degli errori silenziosa, se non si  potuto connettere allora
        disabilitiamo la chat e informiamo l'utente }
        FrmLobbyClient.ShowStatus('Fallito - ' + E.Message,clGray);
      end;
    end;
  end;

  { Se abbiamo finito i gateway, notifica un messaggio di errore...(e qui finisce il thread...) }
  if not ClientSocket.Connected then FrmLobbyClient.ShowStatus('Impossibile connettersi al lobby multiplayer, nessuno dei gateway  disponibile al momento. Riprova pi tardi.',clRed);
end;

{ Procedura per ricevere la lista dei gateway da internet }
procedure TLobbyClient.ReceiveGatewayList;
var
  HttpSocket: TIdHTTP;
  Response: string;
  Host, Port: string;
begin
  { Crea l'oggetto TIdHTTP }
  HttpSocket := TIdHTTP.Create(AOwner);

  { Inizializza la lista dei gateway }
  GatewayList := TList.Create;

  { Esegue la richiesta }                            
  Response := HttpSocket.Get(GATEWAYLISTFILE);

  { Se il file  vuoto, non c' nessun gateway disponibile, altrimenti splitta la risposta }
  if Response = '' then FrmLobbyClient.ShowStatus('Nessun gateway disponibile per la connessione al lobby multiplayer.',clNavy)
  else begin

    { Response = 'Host Port_Host Port_Host Port..' }
    while Trim(Response) <> '' do         
    begin
      { Prende l'host, tronca, prende la porta e tronca, e cos via... }
      Host := AnsiLeftStr(Response,AnsiPos(' ',Response)-1);
      Response := AnsiRightStr(Response,Length(Response)-AnsiPos(' ',Response));
      Port := AnsiLeftStr(Response,AnsiPos(GATEWAYSEPARATOR,Response)-1);
      Response := AnsiRightStr(Response,Length(Response)-AnsiPos(GATEWAYSEPARATOR,Response));

      { Aggiunge il gateway alla lista... }
      AddGateway(Host,Port);
    end;
  end;
end;

{ Procedura per aggiungere un gateway alla lista... }
procedure TLobbyClient.AddGateway(Host: string; Port: string);
begin
  GatewayList.Add(TLobbyGateway.Create(Host,StrToInt(Port)));
  FrmLobbyClient.ShowStatus('Ricevo il gateway ' + Host + ':' + Port);
end;


{ Procedura per inviare un messaggio di chat al lobby }
procedure TLobbyClient.BtnSend_Click(Sender: TObject);
begin
  if FrmLobbyClient.txtMessage.Text <> '' then begin
    ClientSocket.WriteLn('/MSG'+SEPARATOR+FrmLobbyClient.txtMessage.Text);
    FrmLobbyClient.txtMessage.Text := '';
  end;
end;

{ Procedura che viene eseguita al momento della connessione remota con il server }
procedure TLobbyClient.ClientSocket_Connected(Sender: TObject);
var
  StrData: string;
  Command: string;
  C: integer;
  LobbyListChecker: TLobbyListChecker;
  ServerInfoList: TServerInfoList;
begin
  { OKK, abbiamo trovato un gateway! Evvai lol }
  FrmLobbyClient.ShowStatus('Connessione stabilita con ' + ClientSocket.Host + ':' + IntToStr(ClientSocket.Port),clGreen);

  { Abilitiamo i comandi ... }
  FrmLobbyClient.BtnSend.Enabled := true;
  FrmLobbyClient.BtnHost.Enabled := true;

  { Per PRIMA cosa controlliamo che il numero di versione del lobby corrisponda.. }
  ClientSocket.WriteLn('/MYLOBBYVERSIONIS'+SEPARATOR+LOBBYVERSION);
  StrData := ClientSocket.ReadLn;
  if GetCommand(StrData) = '/URLOBBYVERSIONISNOTSUPPORTED' then raise Exception.Create('La versione del server (' + GetToken(StrData,1) + ') differisce dalla nostra (' + LOBBYVERSION + ').');

  { Tutto a posto, chiediamo di loggarci al server }
  ClientSocket.WriteLn('/LOGIN'+SEPARATOR+Nickname);
  
  try
    while ClientSocket.Connected do
    begin
      StrData := ClientSocket.ReadLn;
      Command := GetCommand(StrData);

      { Il server ci avvisa che  in arrivo un messaggio di chat..
      Command = '/MSG_Message'  }
      if Command = '/MSG' then begin
        FrmLobbyClient.AppendMsg(GetToken(StrData,1));
      end

      { Il server ci sta inviando la lista degli utenti
      Command = '/USERLIST_Count_User_User_User...}
      else if Command = '/USERLIST' then begin
        { Azzera la lista }
        FrmLobbyClient.LstUsers.Clear;

        { E la riempie }
        for C := 0 to StrToInt(GetToken(StrData,1))-1 do
          FrmLobbyClient.LstUsers.AddItem(GetToken(StrData,C+2),nil);
      end

      { Il server ci sta inviando la lista delle partite...
      Command = '/GAMELIST_Count_Host_Port_Host_Port...}
      else if Command = '/GAMELIST' then begin

        { Costruisci la lista se c' almeno un game }
        if StrToInt(GetToken(StrData,1)) > 0 then begin

          { Inizializziamo le variabili necessarie... }
          ServerInfoList := TServerInfoList.Create;

          { Riempie la lista dei servers... }
          C := 2;
          while C <= (StrToInt(GetToken(StrData,1))*2+1) do
          begin
            ServerInfoList.AddServerInfo(TServerInfo.Create(AOwner,GetToken(StrData,C),StrToInt(GetToken(StrData,C+1))));
            C := C + 2;
          end;

          { E la analizza... }
          LobbyListChecker := TLobbyListChecker.Create(AOwner,FrmLobbyClient.LstGames,ServerInfoList);
        end
        else begin
          { Cancella la lista se non ci sono games e disabilita il pulsante... }
          FrmLobbyClient.LstGames.Clear;
          FrmLobbyClient.BtnJoin.Enabled := false;
        end;
      end

      { Il server ci sta inviando i risultati del test di connessione...
      NOTA: attenzione all'ordine, non invertirlo... }
      else if Command = '/CONNECTIONTESTRESULT' then begin
        if StrToBool(GetToken(StrData,1)) then
          FrmConnectionTest.SetImageOnControl(FrmConnectionTest.ImgConnectionTest,true)
        else begin
          IAmAbleToHost := false;
          FrmConnectionTest.SetImageOnControl(FrmConnectionTest.ImgConnectionTest,false);
        end;
      end

      else if Command = '/MESSAGETESTRESULT' then begin
        if StrToBool(GetToken(StrData,1)) then
          FrmConnectionTest.SetImageOnControl(FrmConnectionTest.ImgSendReceiveTest,true)
        else begin
          IAmAbleToHost := false;
          FrmConnectionTest.SetImageOnControl(FrmConnectionTest.ImgSendReceiveTest,false);
        end;

        { Ok, fine del test }
        Sleep(2000); //Piccola pausa per non far sfarfallare la finestra
        ConnectionTestDone := true;
        TestServer.StopServer;
        FrmConnectionTest.Hide;

        { Scatena l'evento associato alla fine del test }
        ConnectionTest_Finished(nil);
      end

      { Il server ha cambiato il nostro nickname...
      Command = '/NICKNAMECHANGEDSUCCESSFUL_Nickname' }
      else if Command = '/NICKNAMECHANGEDSUCCESSFUL' then begin
        Nickname := GetToken(StrData,1);
        FrmLobbyClient.Nickname := Nickname;
      end;

    end;
  except
    on E: exception do
    begin
      { Gestione silenziosa degli errori }
      FrmLobbyClient.BtnSend.Enabled := false;
      FrmLobbyClient.BtnHost.Enabled := false;
      raise Exception.Create('Abbiamo perso la connessione con il gateway ' + ClientSocket.Host + ':' + IntToStr(ClientSocket.Port) + ' - ' + E.Message);
    end;
  end;

end;


{ Evento richiamato quando  finito il test }
procedure TLobbyClient.ConnectionTest_Finished(Sender: TObject);
begin
  { Una volta finito il test, apri la finestra per hostare se siamo abilitati a farlo }
  if IAmAbleToHost then begin

    { Ritorna il controllo alla form principale ... }
    FrmLobbyClient.ShowStatus('Test della connessione eseguito con successo! Sei abilitato ad hostare partite sulla rete internet.',clGreen);

    { Ma prima comunica al server di aggiunge la partita all'elenco... }
    ClientSocket.WriteLn('/ISTARTANEWGAME'+SEPARATOR+FrmLobbyClient.HostGame.txtPort.Text);

    FrmLobbyClient.Choice := gcHost;
    FrmLobbyClient.ModalResult := mrOK;

    Disconnect;
  end
  else begin
    FrmLobbyClient.ShowStatus('Test della connessione fallito! Questo fallimento potrebbe essere causato dalla presenza di un firewall installato sul computer locale, oppure dalla presenza di un router nel caso di connessione ad internet per mezzo di LAN switchate.',clRed);
  end;
end;

{ Procedura per hostare un game }
procedure TLobbyClient.BtnHost_Click(Sender: TObject);
begin
  FrmLobbyClient.HostGame := TFrmHostGame.Create(AOwner);
  FrmLobbyClient.HostGame.txtNickname.Text := Nickname;
  FrmLobbyClient.HostGame.txtTableName.Text := Nickname + '''s game';

  if FrmLobbyClient.HostGame.ShowModal = mrOK then begin
    { Prima di autorizzare il client ad avviare una partita, eseguiamo un test per verificare
    l'autorevolezza del client ad hostare un game }
    ConnectionTestDone := false;

    { Supponiamo che il test vada bene.. }
    IAmAbleToHost := true;

    { Crea la form di visualizzazione del test }
    FrmConnectionTest := TFrmConnectionTest.Create(AOwner);
    FrmConnectionTest.lblOpenTest.Caption := 'Apertura del server di test sulla porta ' + FrmLobbyClient.HostGame.txtPort.Text;
    FrmConnectionTest.Show;
    FrmConnectionTest.Refresh;
    FrmLobbyClient.Refresh;

    { Prima di avviare il test fai una pausa per permettere all'utente di leggere
    quel che sta accadendo }
    Sleep(2000);

    { Avvia il server di test... }
    TestServer := TTestServer.Create(AOwner);
    if TestServer.StartServer(StrToInt(FrmLobbyClient.HostGame.txtPort.Text)) then
      FrmConnectionTest.SetImageOnControl(FrmConnectionTest.ImgOpenTest,true)
    else IAmAbleToHost := false;

    { Avvisa il lobby multiplayer di eseguire il test... }
    ClientSocket.WriteLn('/IMREADYFORTHECONNECTIONTEST'+SEPARATOR+FrmLobbyClient.HostGame.txtPort.Text);

    //Togli per generare un errore di connessione... (debug)
    //ClientSocket.WriteLn('/IMREADYFORTHECONNECTIONTEST'+SEPARATOR+IntToStr(1400));
  end;
end;

{ L'utente ha deciso di partecipare ad una partita esistente... }
procedure TLobbyClient.BtnJoin_Click(Sender: TObject);
begin
  FrmLobbyClient.Choice := gcJoin;
  FrmLobbyClient.ModalResult := mrOK;
  Disconnect;
end;


end.
