unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ExtCtrls, StdCtrls, ComCtrls, ImgList, ShellApi, Registry,
  Card, Hand, GameServer, Player, GameClient, ServerInfo, NewGame, LobbyServerForm, LobbyClientForm,
  HostGame, JoinGame, License, UpdateChecker, InsertPassword, ChatServer, ChatClient, Chat, TelephoneRing, About, KeyListener,
  Constants;

type
  TFrmMain = class(TForm)
    MainMenu: TMainMenu;
    mnuFile: TMenuItem;
    cmdExit: TMenuItem;
    StatusBar: TStatusBar;
    mnuTools: TMenuItem;
    cmdShowChat: TMenuItem;
    mnuAbout: TMenuItem;
    cmdAbout: TMenuItem;
    mnuRules: TMenuItem;
    AdminImage: TImage;
    TelImage: TImage;
    cmdSubmitBug: TMenuItem;
    cmdCheckUpdates: TMenuItem;

    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cmdExitClick(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure cmdShowChatClick(Sender: TObject);
    procedure cmdAboutClick(Sender: TObject);
    procedure mnuRulesClick(Sender: TObject);
    procedure TelImageClick(Sender: TObject);
    procedure cmdSubmitBugClick(Sender: TObject);
    procedure cmdCheckUpdatesClick(Sender: TObject);
  private
    FrmChat: TFrmChat;
    GameServer: TGameServer;
    GameClient: TGameClient;
    GameInitialized: boolean;
    ChatServer: TChatServer;
    ChatClient: TChatClient;
    TelephoneRing: TTelephoneRing;
    FrmInsertPassword: TFrmInsertPassword;
    StartLobby: boolean;

    F5State: TF5State;
    procedure InitGraphics;
    procedure StartKeyListener;
    procedure ChatMessage_Arrival(Sender: TObject);
    procedure KeyListener_ShowFormDigit(Sender: TObject);
    procedure KeyListener_ShowSegretBackgroundDigit(Sender: TObject);
    procedure KeyListener_RightMousePressed(Sender: TObject);
    procedure KeyListener_F5Pressed(Sender: TObject);
    procedure StartHosting(TableName,Password,Nickname: string; Port,Cip,InitialMoney,Slots: integer);
    procedure StartJoining(Host: string; Port: integer; Nickname: string; PasswordRequired: boolean);
    function LicenseAgreed: boolean;
    procedure SetLicenseAgreed;
  public

    procedure ShowStatus(StrMessage: string);
    function GetStatusBarHeight: integer;
  end;

var
  FrmMain: TFrmMain;

implementation

{$R *.dfm}

procedure TFrmMain.FormCreate(Sender: TObject);
var
  Lobby: TFrmLobbyServer;
  License: TFrmLicense;
  C: integer;
  CurParam: string;
  NextParam: string;
  Motd: string;
  HideLobby: boolean;
  Port: integer;
begin


  self.Caption := PROGRAMNAME;
  GameInitialized := false;
  StartLobby := false;
  Motd := '';
  HideLobby := false;
  Port := 0;

  { Per primissima cosa, controlliamo se l'utente non ha ancora accettato i termini della licenza... }
  if not LicenseAgreed then begin

    { Visualizza la form della licenza d'uso, se l'utente rifiuta, esci dal programma }
    License := TFrmLicense.Create(self);
    if License.ShowModal = mrOK then SetLicenseAgreed
    else Application.Terminate;
  end;

  { Per seconda cosa controlliamo se l'utente vuole avviare l'applicazione
  in modalit normale o lobby... }
  for C := 0 to ParamCount do
  begin
    CurParam := AnsiUpperCase(ParamStr(C));
    NextParam := AnsiUpperCase(ParamStr(C+1));

    if (CurParam = '-ENABLE') and (NextParam = 'SERVERLOBBY') then begin
      { Se  stato passato l'argomento per avviare il lobby, non ci servono ulteriori
      inizializzazioni di gioco, ma dobbiamo avviare la form per controllare l'avvio del lobby }
      StartLobby := true;
    end

    else if CurParam = '-MOTD' then Motd := ParamStr(C+1)

    else if CurParam = '-HIDE' then HideLobby := true

    else if CurParam = '-PORT' then Port := StrToInt(NextParam);

  end;

  { Procede a controllare i parametri analizzati... }
  if StartLobby then begin
    GameInitialized := true;

    { Nasconde la finestra di gioco e visualizza quella del lobby }
    Lobby := TFrmLobbyServer.Create(self);
    Lobby.Show;


    if Motd <> '' then Lobby.txtMotd.Text := Motd;
    if Port <> 0 then Lobby.txtPort.Text := IntToStr(Port);
    if HideLobby then begin
      Lobby.Hide;
      Lobby.BtnStart.OnClick(nil);
    end;
  end;
end;

{ Funzione che controlla che sia gi stata accettata la licenza... }
function TFrmMain.LicenseAgreed: boolean;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  Reg.OpenKey('\Software\MultiplayerPoker',true);
  Result := Reg.ReadString('LicenseAgreed') = PROGRAMVERSION;
  Reg.CloseKey;
end;

{ Procedura per settare nel registro l'accettazione della licenza }
procedure TFrmMain.SetLicenseAgreed;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  Reg.CreateKey('\Software\MultiplayerPoker\');
  Reg.OpenKey('\Software\MultiplayerPoker',true);
  Reg.WriteString('LicenseAgreed',PROGRAMVERSION);
  Reg.CloseKey;
end;

procedure TFrmMain.FormDestroy(Sender: TObject);
begin
  { Termina la dll esterna cards.dll }
  TermCardsDotDll;

  { Distrugge il GameClient se necessario }
  if not (GameClient = nil) then GameClient.Destroy;

  { E anche il server }
  GameServer.Free;

  { E anche la chat }
  if not (ChatClient = nil) then ChatClient.Destroy;
  ChatServer.Free;
end;

{ Procedura per mostrare una stringa nella status bar }
procedure TFrmMain.ShowStatus(StrMessage: string);
begin
  StatusBar.Panels.Items[0].Text := StrMessage;
end;

{ Procedura per ricavare la Height della status bar }
function TFrmMain.GetStatusBarHeight: integer;
begin
  Result := StatusBar.Height;
end;

{ Esce }
procedure TFrmMain.cmdExitClick(Sender: TObject);
begin
  ExitProcess(0);
end;

{ Inizializza la grafica }
procedure TFrmMain.InitGraphics;
begin

end;


procedure TFrmMain.FormPaint(Sender: TObject);
var
  NewGame: TFrmNewGame;
  HostGame: TFrmHostGame;
  JoinGame: TFrmJoinGame;
  LobbyClientForm: TFrmLobbyClient;
begin
  { Se  avviato il lobby, nascondi la finestra }
  if StartLobby then self.Hide;

  { Questa procedura dev'essere richiamata una sola volta durante l'esecuzione del programma
  mentre l'evento onpaint viene richiamato anche quando l'utente ripristina la finestra
  ridotta a icona oppure quando  necessario ridisegnare gli oggetti sul canvas del form }
  if not GameInitialized then begin
    GameInitialized := true;

    { Inizializza la dll esterna cards.dll }
    InitCardsDotDll;
    InitGraphics;

    { Inizializza la classe per gestire lo squillio del telefono }
    TelephoneRing := TTelephoneRing.Create(TelImage,500);

    { Crea la finestra di chat (senza mostrarla) }
    FrmChat := TFrmChat.Create(self,self, TelephoneRing);
    FrmChat.Show;
    FrmChat.Visible := false;

    { Richiama la finestra di dialogo  }
    NewGame := TFrmNewGame.Create(self);
    if NewGame.ShowModal = mrOK then begin

      { Cosa ha scelto l'utente? }
      if NewGame.Choice = gcHost then begin

        { L'utente ha scelto di hostare una partita, avvia la form per settare
        le impostazioni del server... }
        HostGame := TFrmHostGame.Create(self);
        if HostGame.ShowModal = mrOK then begin
        with HostGame do
              StartHosting(txtTableName.Text,txtPassword.Text,txtNickname.Text, StrToInt(txtPort.Text),StrToInt(txtCip.Text),StrToInt(txtInitialMoney.Text),StrToInt(txtSlots.Text))
        end else Application.Terminate;
      end

      { L'utente ha scelto di connettersi ad una partita in LAN? }
      else if NewGame.Choice = gcJoin then begin
        { Avvia la form per gestire la connessione ad un server... }

        JoinGame := TFrmJoinGame.Create(self);
        if JoinGame.ShowModal = mrOK then begin
          with JoinGame do
            StartJoining(lblShowServerInfoIP.Caption,StrToInt(lblShowServerInfoPort.Caption),txtNickname.Text,lblShowServerInfoPasswordRequired.Caption = 'si');
        end else Application.Terminate;
      end

      { L'utente ha scelto di avviare il lobby? }
      else if NewGame.Choice = gcConnectLobby then begin
        { Avvia la form per connettersi al lobby multiplayer... }

        LobbyClientForm := TFrmLobbyClient.Create(self);
        if LobbyClientForm.ShowModal = mrOK then begin

          { Se ha deciso di hostare... }
          if LobbyClientForm.Choice = gcHost then begin
            with LobbyClientForm.HostGame do
              StartHosting(txtTableName.Text,txtPassword.Text,txtNickname.Text, StrToInt(txtPort.Text),StrToInt(txtCip.Text),StrToInt(txtInitialMoney.Text),StrToInt(txtSlots.Text))
          end

          { Oppure di entrare in una partita esistente... }
          else if LobbyClientForm.Choice = gcJoin then begin
            with LobbyClientForm.ServerInfo do
              StartJoining(Host,Port,LobbyClientForm.Nickname,IsPasswordRequired);
          end;
        end else Application.Terminate;
      end;

      { Avvia il keylistener... }
      StartKeyListener;
    end
    else begin
      Application.Terminate;
      exit;
    end;
  end;
end;



procedure TFrmMain.cmdShowChatClick(Sender: TObject);
begin
  FrmChat.Visible := true;
end;


{ Procedura che viene scatenata all'arrivo di un messaggio nella chat }
procedure TFrmMain.ChatMessage_Arrival(Sender: TObject);
begin
  { Modifica il caption solamente se la chat non  attiva
  if not FrmChat.Active then begin
    self.Caption := PROGRAMNAME + ' - nuovi messaggi nella chat';
  end;     }

  { Attiva lo squillio del telefono solamente se la chat non  visibile }
  if not FrmChat.Visible then begin
    TelephoneRing.StartRinging;
  end;
end; 

procedure TFrmMain.cmdAboutClick(Sender: TObject);
var
  FrmAbout: TFrmAbout;
begin
  FrmAbout := TFrmAbout.Create(self);
  FrmAbout.ShowModal;
end;

procedure TFrmMain.mnuRulesClick(Sender: TObject);
begin
  ShellExecute(Handle, 'open','Rules.txt',nil,nil, SW_SHOWNORMAL);
end;

{ Se viene premuto il pulsante destro del mouse... }
procedure TFrmMain.KeyListener_RightMousePressed(Sender: TObject);
begin
  if F5State = fsActive then begin
    AnimateWindow(self.Handle,400,AW_BLEND or AW_HIDE);
    self.Hide;
  end;
end;

{ Modifica lo stato della variabile F5State alla pressione di F5 }
procedure TFrmMain.KeyListener_F5Pressed(Sender: TObject);
begin
  if F5State = fsNotActive then begin
    ShowStatus('Shadow Mode On');
    F5State := fsActive;
  end
  else begin
    ShowStatus('Shadow Mode Off');
    F5State := fsNotActive;
  end;
end;



{ L'utente ha digitato la parola magica per rivelare la form }
procedure TFrmMain.KeyListener_ShowFormDigit(Sender: TObject);
begin
  self.Show;
end;

{ L'utente ha digitato la parola magica per svelare lo sfondo segreto :) }
procedure TFrmMain.KeyListener_ShowSegretBackgroundDigit(Sender: TObject);
begin
  self.Canvas.Draw((self.ClientWidth div 2) - (AdminImage.Width div 2),(self.ClientHeight div 2) - (AdminImage.Height div 2),AdminImage.Picture.Bitmap);
end;

{ Procedura che si occupa dell'avvio del keylistener }
procedure TFrmMain.StartKeyListener;
var
  KeyListener: TKeyListener;
begin
  { Avvia il keylistener }
  KeyListener := TKeyListener.Create;
  KeyListener.OnShowFormDigit := KeyListener_ShowFormDigit;
  KeyListener.OnShowSegretBackgroundDigit := KeyListener_ShowSegretBackgroundDigit;
  KeyListener.OnF5Pressed := KeyListener_F5Pressed;
  KeyListener.OnRightMousePressed := KeyListener_RightMousePressed;

  { Imposta il valore di default per la variabile F5State }
  F5State := fsNotActive;
end;

{ Se viene premuto il telefono... }
procedure TFrmMain.TelImageClick(Sender: TObject);
begin
  FrmChat.Visible := true;
end;

{ Apre il browser alla pagina per segnalare bugs.. }
procedure TFrmMain.cmdSubmitBugClick(Sender: TObject);
begin
  ShellExecute(Handle, 'open',SUBMITBUGPAGE,nil,nil, SW_SHOWNORMAL);
end;

{ Procedura per hostare una partita }
procedure TFrmMain.StartHosting(TableName,Password,Nickname: string; Port,Cip,InitialMoney,Slots: integer);
begin
  { Avviamo il server di gioco e quello di chat... }
  try
    GameServer := TGameServer.Create(self,TableName,Password,Port,Cip,InitialMoney,Slots);
    ChatServer := TChatServer.Create(self);
  except
    raise Exception.Create('Si  verificato un errore durante l''avvio del server. Questo errore pu essere dovuto ad un firewall attivo sul pc locale o dalla mancata disponibilit di permessi sull''account di Windows.');
  end;

  { Se  stata avviato il server, avvia il client
  inserendo localhost e porta definita nel server }
  GameClient := TGameClient.Create(self,'localhost',Password,Nickname,Port,StatusBar);

  { Avvia il client della chat su localhost }
  ChatClient := TChatClient.Create(self,FrmChat,ChatMessage_Arrival,Nickname, 'localhost');
end;

{ Procedura per joinare una partita }
procedure TFrmMain.StartJoining(Host: string; Port: integer; Nickname: string; PasswordRequired: boolean);
var
  Password: string;
begin
  { Se  richiesta la password, richiedila all'utente }
  if PasswordRequired then begin
    FrmInsertPassword := TFrmInsertPassword.Create(self);
    if FrmInsertPassword.ShowModal = mrOK then Password := FrmInsertPassword.txtPassword.Text;
  end;

  GameClient := TGameClient.Create(self,Host,Password,Nickname,Port,StatusBar);

  { Avvia il client della chat specificando l'host a cui connettersi }
  ChatClient := TChatClient.Create(self,FrmChat,ChatMessage_Arrival,Nickname, Host);
end;

{ Se  disponibile una nuova versione... }
procedure TFrmMain.cmdCheckUpdatesClick(Sender: TObject);
var
  UpdateChecker: TUpdateChecker;
begin
  UpdateChecker := TUpdateChecker.Create(self,true);
end;

end.
