unit PlayerList;

interface

uses Windows, Classes, SysUtils, Player,
  Constants;

type
  TPlayerList = class(TList)
  private
    MaxPlayers: integer;
  public
    constructor Create(MaxPlayers: integer);

    procedure AddPlayer(Player: TPlayer);
    function GetPlayer(Index: integer): TPlayer;
    function GetPlayerWithId(Id: integer): TPlayer;
    procedure Clear; override;
  end;

implementation

{ Implementazione classe TPlayerList }
constructor TPlayerList.Create(MaxPlayers: integer);
begin
  self.MaxPlayers := MaxPlayers;
end;

{ Procedura che inserisce un client nella lista }
procedure TPlayerList.AddPlayer(Player: TPlayer);
begin
  if self.Count < MaxPlayers then
    self.Add(Player)
  else raise Exception.Create('Si  tentato di aggiungere un elemento di troppo nella lista dei giocatori. Contattare il produttore.');
end;

{ Funzione che restituisce un oggetto TPlayer in base all'index }
function TPlayerList.GetPlayer(Index: integer): TPlayer;
begin
  Result := TPlayer(self.Get(Index));
end;

{ Funzione che restituisce un oggetto TPlayer in base all'id }
function TPlayerList.GetPlayerWithId(Id: integer): TPlayer;
var
  C: integer;
  Player: TPlayer;
begin
  for C := 0 to self.Count-1 do
  begin
    Player := GetPlayer(C);
    if Player.Id = Id then begin
      Result := Player;
      exit;
    end;
  end;

  { Non  stato trovato nessun giocatore con questo id, scatena un eccezzione }
  raise Exception.Create('Si  cercato di recuperare dalla lista il giocatore con Id: ' + IntToStr(Id) + ', ma non  stato trovato. Contattare il produttore');
end;

{ Procedura che elimina tutti i giocatori
(prima di eliminarli, li nasconde per non visualizzarli nel form) }
procedure TPlayerList.Clear;
var
  C: integer;
begin
  for C := 0 to self.Count-1 do
    GetPlayer(C).Hide;

  inherited Clear;
end;

end.
 