unit UpdateChecker;

interface

uses Windows, Classes, SysUtils, ShellApi, IdHTTP, StrUtils,
  Constants;

type
  TUpdateChecker = class(TThread)
  private
    AOwner: TComponent; //Riferimento al componente padre...
    IdHTTP: TIdHTTP;
    NotifyNoNewVersion: boolean;

    function ReadLnFromStr(var Str: string): string;
  public
    constructor Create(AOwner: TComponent; NotifyNoNewVersion: boolean);
    procedure Execute; override;
  end;

implementation

{ Implementazione classe TUpdateChecker }
constructor TUpdateChecker.Create(AOwner: TComponent; NotifyNoNewVersion: boolean);
begin
  self.AOwner := AOwner;
  self.NotifyNoNewVersion := NotifyNoNewVersion;

  { Inizializza il controllo IdHTTP e richiama il costruttore base
  per avviare il thread }
  IdHTTP := TIdHTTP.Create(AOwner);

  inherited Create(false);
end;

{ Entry Point }
procedure TUpdateChecker.Execute;
var
  Response: string;
  NewVersionFound: boolean;
begin
  try
    Response := IdHTTP.Get(UPDATEFILE);

    { Se la nostra versione  differente da quella online... new version found XD! }
    NewVersionFound := Trim(ReadLnFromStr(Response)) <> PROGRAMVERSION;
  except
    NewVersionFound := false;
  end;


  { Se c' una nuova versione, chiedi conferma all'utente... }
  if NewVersionFound then begin

    { Se l'utente acconsente ad aggiornarsi, avviamo il programma di aggiornamento... }
    if MessageBox(0,'E'' stata trovata una versione pi recente della nostra. Si consiglia di tenere sempre il MultiplayerPoker aggiornato all''ultima versione dal momento che le versioni pi recenti possiedono meno errori e pi funzionalit. Aggiornare ora?','Nuovi aggiornamenti disponibili',MB_YESNO or MB_ICONQUESTION) = ID_YES then begin

      ShellExecute(Handle, 'open','Update.exe','-POKERVERSION ' + PROGRAMVERSION,nil,SW_SHOWNORMAL);

      { E inviamo la richiesta al chiamante di chiudere l'applicazione }
      ExitProcess(0);
    end;
  end

  { Altrimenti se richiesto, avvisa l'utente che non c' nessuna nuova versione... }
  else if NotifyNoNewVersion then MessageBox(0,'Il programma  gi aggiornato all''ultima versione.','Non sono necessari aggiornamenti',MB_ICONINFORMATION);
end;

{ Funzione che legge una riga da una stringa e ritorna tale riga come valore
di ritorno, contemporaneamente tronca la stringa (from Updater by Piero Tofy) }
function TUpdateChecker.ReadLnFromStr(var Str: string): string;
var
  C: integer;
begin
  Result := '';
  
  { Esegui queste operazioni solo se la stringa esiste... }
  if Length(Str) > 0 then begin
    C := 1;
    while (Str[C] <> #0) and (Str[C] <> #10) do
    begin
      Result := Result + Str[C];
      inc(C);
    end;
    Str := AnsiRightStr(Str,Length(Str)-C);
  end;
end;

end.
 